function [FinalF0Est, FinalBEst, featureVector, M] = icf(bEst,file,OffsetPercentage)

    [ guitar, sampRate] = audioread( file );
%     [ guitar, sampRate] = readAudioMono( file );
    %% Read in a wav file including:
    %% - piano (the signal)
    %% - sampRate (the sample rate (Hz))
    %% - noBits (No of bits used to sample the data)

    % Determine the length of the fft
    [guitar_length, n] = size( guitar );                           
    FFT_length = 1;
    while FFT_length < guitar_length
        FFT_length = FFT_length * 2;
    end
    
    %% Compute the FFT of the sample using the length
    %% obtained above. The resulting vector is padded
    %% with zeros.
    FFT_guitar = fft( guitar, FFT_length );

    %% Compute the power spectrum
    %% Only need to plot range of interest
    powerSpect = FFT_guitar.* conj(FFT_guitar) / FFT_length;
    FFT_range = round( FFT_length / 2);
    f = sampRate*(0:FFT_range)/FFT_length;

    %% -------------------------------------------------------- %%
    %% --------------- First Step of Algorithm ---------------- %%
    %% -------------------------------------------------------- %%

    %% M Pass Bands
    M = 30;

    %% f0Est over the frequency range in 200
    %% logarithmically spaced steps.
    First = log( 20 );
    Last = log( 1500 );
    Testing = linspace( First, Last, 200 );

    %% In 1st run hold inharmonicity coefficient
    %% at an intermediate value

    MaxSum = -10000;
    %% Vary f0 over the full frequency range in 200 steps
    for n = 1:200

        f0 = exp( Testing( n ) );
        FSUM = 0;
        FSUM2 = [];
        
        M  = 30;
        point = 10000000000;
        FreqSum = 0;
        Offset  = f0 * OffsetPercentage;

        while point >= length(powerSpect)

            x = 1 + ( ( M^2 ) * bEst );
            fK = M * f0 * sqrt( x );

            J = floor( fK + Offset );
            point = FFT_length * ( ( J ) / sampRate );

            M = M - 1;
        end    
        
        for Pass = 1:M
            %% Compute the frequency of band k
            x = 1 + ( ( Pass^2 ) * bEst );
            fK = Pass * f0 * sqrt( x );
            %% Sum output of Comb Filter
            %% Range is 10% either side of f0
            
            FreqSum = 0;
            Offset = f0 * OffsetPercentage;
            
            for J = floor( fK - Offset ) : floor( fK + Offset )
                point = FFT_length * ( ( J ) / sampRate ); 
                FreqSum = FreqSum + powerSpect( round( point ) );
            end
            %% Sum out put from all pass bands
            FSUM = FSUM + FreqSum;
            FSUM2(Pass) = FreqSum;
        end
        %% Find the max FSUM
        if FSUM > MaxSum
            MaxSum = FSUM;
            fEst1 = f0;
        end    
    end
    clear First Last Testing;

    %% --------------------------------------------------------- %%
    %% --------------- Second Step of Algorithm ---------------- %%
    %% --------------------------------------------------------- %%

    %% f0Est over frequency range in 30
    %% linearly spaced steps.
    First = fEst1 * 0.9;
    Last = fEst1 * 1.1;
    Testing = linspace( First, Last, 30 );

    %% BEst over its full range in 200 steps
    %% Logarithmically spaced between 0 & 0.001
    First1 = log( 50 * 10^(-6) );
    Last1 = log( 0.001 );
    Testing1 = linspace( First1, Last1, 200 );

    MaxSum = -10000;
    for q = 1:200
        bEst = exp( Testing1( q ) );
        for n = 1:30
            f0 = Testing( n );
            FSUM = 0;
            for Pass = 1:M
                %% Compute the frequency of band k
                x = 1 + ( ( Pass^2 ) * bEst );
                fK = Pass * f0 * sqrt( x );
                FreqSum = 0;
                                                %% Sum output of filter
                Offset = f0 * 0.2;
                for K = floor( fK - Offset ) : floor( fK + Offset )
                    point = FFT_length * ( ( K ) / sampRate );
                    FreqSum = FreqSum + powerSpect( floor(point) );
                end
                FSUM = FSUM + FreqSum;
            end

            if FSUM > MaxSum
                MaxSum = FSUM;
                fEst2 = f0;
                bEstimate2 = bEst;
            end    
         end
    end
    clear First First1 Last Last1 Testing Testing1;

    %% ------------------------------------------------------- %%
    %% ------------- Third Step of Algorithm ----------------- %%
    %% ------------------------------------------------------- %%

    %% f0Est over the frequency range in 30
    %% linearly spaced steps.
    
    First = fEst2 * 0.95;
    Last = fEst2 * 1.05;
    Testing = linspace( First, Last, 30 );

    %% BEst over its full range in 30 steps
    %% Logarithmically spaced between 0 & 0.001
    First1 = bEstimate2 * 0.95;
    Last1 = bEstimate2 * 1.05;
    Testing1 = linspace( First1, Last1, 30 );
    
    Tracker = zeros(30,30);
    MaxSum = -10000;

    featureVector = zeros(1,M);
%     M= 30;
    
    for q = 1:30
        bEst = Testing1( q );
        for n = 1:30
            f0 = Testing( n );
            FSUM = 0;
            
            
            s = 1;
            for Pass = 1:M
                %% Compute the frequency of band k
                x = 1 + ( ( Pass^2 ) * bEst );
                fK = Pass * f0 * sqrt( x );
                FreqSum = 0;
                Offset = f0 * 0.1;
                for K = floor( fK - Offset ) : floor( fK + Offset )
                    point = FFT_length * ( ( K ) / sampRate );
                    FreqSum = FreqSum + powerSpect( floor(point) );
                end
                FSUM = FSUM + FreqSum;
                featureVector(s) = FreqSum;
                s = s + 1;
            end

            Tracker(n,q) = FSUM;
            if FSUM > MaxSum;
                MaxSum = FSUM;
                fEst3 = f0;
                bEstimate3 = bEst;
                MaxN = n;
                MaxQ = q;
            end    
        end
    end

    %% --------------------------------------------------------- %%
    %% ---------------- Interplate the Results ----------------- %%
    %% --------------------------------------------------------- %%

    
    %%  Default values set to zero to stop out of bounds issues
    % This stops linear interpolation near edges of matrix.
    
    interpMinus2 = 0;
    interpMinus1 = 0;
    
    interpPlus1 = 0;
    interpPlus2 = 0;
    
    if MaxN >= 3
        interpMinus2 = 2;
        interpMinus1 = 1;
    end
    
    if MaxN <= 28
        interpPlus1 = 1;
        interpPlus2 = 2;
    end

    %% Find final F0 Estimate
    StepSizeF = Testing(3) - Testing(2);
    MinusTwo = Tracker(MaxN - interpMinus2,MaxQ);
    MinusOne = Tracker(MaxN - interpMinus1,MaxQ);
    Middle = Tracker(MaxN,MaxQ);
    PlusOne = Tracker(MaxN + interpPlus1,MaxQ);
    PlusTwo = Tracker(MaxN + interpPlus2,MaxQ);
    InterpolTop = (-2*MinusTwo^2)-(MinusOne^2)+(PlusOne^2)+(2*PlusTwo^2);
    InterpolBot = (MinusTwo^2)+(MinusOne^2)+(Middle^2)+(PlusOne^2)+(PlusTwo^2);
    FinalF0Est = fEst3 + ((InterpolTop/InterpolBot)*StepSizeF);
    
    clear Minus* Plus* Middle Interpol*
    
    interpMinus2 = 0;
    interpMinus1 = 0;
    
    interpPlus1 = 0;
    interpPlus2 = 0;
    
    if MaxQ >= 3
        interpMinus2 = 2;
        interpMinus1 = 1;
    end
    
    if MaxQ <= 28
        interpPlus1 = 1;
        interpPlus2 = 2;
    end

    %% Find Final estimate of B
    StepSizeB = Testing1(3) - Testing1(2);
    MinusTwo = Tracker(MaxN,MaxQ - interpMinus2);
    MinusOne = Tracker(MaxN,MaxQ - interpMinus1);
    Middle = Tracker(MaxN,MaxQ);
    PlusOne = Tracker(MaxN,MaxQ + interpPlus1);
    PlusTwo = Tracker(MaxN,MaxQ + interpPlus2);

%     MinusTwo = 0; PlusTwo = 0;
%     MinusOne = 0; PlusOne = 0;
    InterpolTop = (-2*MinusTwo^2)-(MinusOne^2)+(PlusOne^2)+(2*PlusTwo^2);
    InterpolBot = (MinusTwo^2)+(MinusOne^2)+(Middle^2)+(PlusOne^2)+(PlusTwo^2);
    FinalBEst = bEstimate3 + ((InterpolTop/InterpolBot)*StepSizeF);
    
end

