%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% This function can predict the string and fret position of a note on a
% guitar.
%
% Input(s): Sample that is to have its position predicted
%           Estimated inharmonicity for the given class of guitar
%           ICF Bin offset
%           Maximum value for linear relationship
%           Minimum value for linear relationship
%           Range (number of notes) of notes to predict from
%                  
% Output(s): Array containing string and fret position
%          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function predictedFretValues = predictPosition(sample, BEst, icfBinOffset, maximumInharmonicity, minimumInharmonicity,numberOfPossibleNotes) 

    [FinalF0Est, FinalBEst, featureVector, M] = icf(BEst, sample, icfBinOffset);
    totalEnergy           = sum(normalise(featureVector,1));
    possibleNeckPositions = findPossibleNotePositions(FinalF0Est);

    estimateFretPossibilities = linspace(maximumInharmonicity,minimumInharmonicity,numberOfPossibleNotes);

    tmp = abs(estimateFretPossibilities - totalEnergy);
    [idx idx] = min(tmp);
    predictedFretPosition = idx; %closest value in to match on neck

    tmp = abs(possibleNeckPositions(:,2)' - predictedFretPosition); %closest value in possible positions
    [idx idx] = min(tmp);

    predictedFretValues      = zeros(1,2);
    predictedFretValues(1,1) = possibleNeckPositions(idx,1);
    predictedFretValues(1,2) = possibleNeckPositions(idx,2) - 1; %-1 to translate matlab indexing to guitar fret positions

end