[guitar Fs] = audioread('../organised_samples/pieces/1_mono.wav');

icfBinOffset    = 0.05;
desiredFeatureVecotrLength = 30;

% Inharmonicity coefficients
stringDiameters = [0.25 0.33 0.43 0.76 1.066 1.3208]; 
stringTensions  = [7.36 6.98 7.52 11.22 11.61 9.59];

opt  = 3;
d = mean(stringDiameters);
t = mean(stringTensions);
ym   = 200;                            % youngsModulus   in GPa
l    = calcFrettedStringLength(610,0); % string length from played fret in mm
BEst = inharmonicity(d,ym,l,t); 

windowsSize = 10000;

numberOfWindows = int64(floor(length(guitar)/windowsSize));

totalEnergy = zeros(1, numberOfWindows);
featureVectorArr = zeros(numberOfWindows, desiredFeatureVecotrLength + 1); 

i = 1;
while i < length(guitar)
    currentWindow = guitar(i:i + windowsSize);

    [FinalF0Est, FinalBEst, featureVector, M] = icfArr(BEst, currentWindow, Fs, icfBinOffset);

    if length(featureVector) < desiredFeatureVecotrLength
        featureVector = padWithZeros(featureVector, desiredFeatureVecotrLength);
    end

    featureVector = cat(2, FinalF0Est, normalise(featureVector,1));

    featureVectorArr(i,:) = featureVector;

    i = i + windowsSize;
end

for i = 1:numberOfWindows
    totalEnergy(i) = sum(featureVectorArr(i,2:end));
end

predictedFretValues = zeros(numberOfWindows,2);

for i = 1:length(filesFiltered)
    possibleNeckPositions = findPossibleNotePositions(featureVectorArr(i,1));

    estimateFretPossibilities = linspace(2,1,13);

    val = totalEnergy(i);
    tmp = abs(estimateFretPossibilities - val);
    [idx idx] = min(tmp);
    predictedFretPosition = idx; %closest value in to match on neck

    tmp = abs(possibleNeckPositions(:,2)' - predictedFretPosition);
    [idx idx] = min(tmp);

    predictedFretValues(i,1) = possibleNeckPositions(idx,1);
    predictedFretValues(i,2) = possibleNeckPositions(idx,2);
end

